<?php
/**
 * @file
 * Contains \Drupal\dblog_pager\Controller\DblogPagerController.
 */

namespace Drupal\dblog_pager\Controller;

use Drupal\Core\Url;
use Drupal\dblog\Controller\DbLogController;

/**
 * Returns response for dblog_pager overriden route (dblog.event).
 */
class DblogPagerController extends DbLogController {

  /**
   * Function: get link to log record as found via the query args/filter.
   *
   * @param array $query_args
   *   Used to generate a where based on record.wid. It must have these indices:
   *     sort => ASC|DESC - the sort direction of the query.
   *     operator => string -- the operator (=/>/< etc.) to be used in the query
   *     value => int -- the value to compare to record.wid.
   * @param array $filter
   *   An array of Where conditions and associative array of values.
   * @param bool $id
   *   Controls what return value type is.
   *
   * @return array|int
   *   if bool $id  is TRUE, return the ID, otherwise return formatted link.
   */
  private function getLink(array $query_args, array $filter = NULL, $id = FALSE) {
    $pos = 0;
    $query = db_select('watchdog', 'w');
    $query
      ->fields('w', array('wid'))
      ->orderBy('w.wid', $query_args['sort']);
    if (!empty($filter['where'])) {
      $pos = count($filter['args']);
      $query->where($filter['where'], $filter['args']);
    }
    $query->where("w.wid {$query_args['operator']} ?",
      array($pos => $query_args['value']));
    $records = $query
      ->range(0, 1)
      ->execute();
    if (!$records) {
      return FALSE;
    }
    foreach ($records as $record) {
      if ($record->wid === $id) {
        continue;
      }
      if ($id === TRUE) {
        return $record->wid;
      }
      return \Drupal::l($query_args['title'], Url::fromRoute('dblog.event', array('event_id' => $record->wid)));
    }
  }

  /**
   * Displays details about a specific database log message.
   *
   * @param int $event_id
   *   Unique ID of the database log message.
   *
   * @return array
   *   If the ID is located in the Database Logging table, a build array in the
   *   format expected by drupal_render();
   */
  public function evDetails($event_id) {
    $output = array();
    $build = $this->eventDetails($event_id);
    $config = \Drupal::config('dblog_pager.settings');
    $filter = $this->buildFilterQuery();

    // Original implementation returns a empty array when log record not found.
    if (!$build) {
      // If we aren't overriding that we'll return an empty array, else warn.
      if (!$config->get('bad_id_override')) {
        return $build;
      }
      drupal_set_message(t('No event log record found for id: @id.',
       array('@id' => $event_id)), 'warning');
      if ($config->get('bad_id_last')) {
        $id = $this->getLink(array(
          'sort' => 'DESC',
          'operator' => '<',
          'value' => PHP_INT_MAX), $filter, TRUE);
        if (!$id) {
          drupal_set_message(t('There appears to be no log messages.'), 'error');
          return '';
        }
        drupal_set_message(t('Selected last log record to display.'), 'status');
        return $this->redirect('dblog.event', array('event_id' => $id));
      }
      else {
        $output['table'] = FALSE;
      }
    }
    else {
      $output['table'] = $build['dblog_table'];
    }
    // Add the pager links.
    $query_data = array(
      'prev' => array(
        'sort' => "DESC",
        'operator' => '<',
        'value' => $event_id,
        'title' => t('Previous')),
      'next' => array(
        'sort' => 'ASC',
        'operator' => '>',
        'value' => $event_id,
        'title' => t('Next')),
    );
    if ($config->get('show_first_last')) {
      $query_data = array(
        'first' => array(
          'sort' => 'ASC',
          'operator' => '>',
          'value' => 0,
          'title' => t('First'),
          ),
        ) + $query_data;
      $query_data['last'] = array(
        'sort' => 'DESC',
        'operator' => '<',
        'value' => PHP_INT_MAX,
        'title' => t('Last'));
    }
    $output['links'] = array(
      '#theme' => 'item_list',
      '#items' => array(),
      '#attributes' => array('class' => 'dblog-event-pager'),
    );
    foreach ($query_data as $query_args) {
      $link = $this->getLink($query_args, $filter, $event_id);
      if (!$link) {
        continue;
      }
      $output['links']['#items'][] = $link;
    }
    $output['#attached'] = array(
      'library' => array('dblog_pager/styling'));
    return $output;
  }

}
